; 
; Beta computer monitor
; 
; Steven Bolt
; 1984 
; Pitronics
;

DRA 	= $80 				; 6532 data register data port A
DRB 	= $82 				; port B
DDRA 	= $81					; data direction A port
DDRB 	= $83 				; port B
;
TEMP 	= $00
KCODE = $04					; command key 
DPTR 	= $05 				; digit position
ADHI 	= $03 				; address
ADLO 	= $02 
DATA 	= $01					; data register
LET1 	= $05
LLET 	= $06
IPCL 	= $0B 				; Interrupt save area
IACC 	= $0C
IFLAG = $0D
IXX 	= $0E			
IYY 	= $0F
ISS 	= $10
IDDRA = $11
IDRA 	= $12
IDDRB = $13
IDRB 	= $14
IRQLO = $07					; irq vector
IRQHI = $08
NMILO = $09					; nmi vector
NMIHI = $0A
SADLO = $0B					; address regsiter
SADHI = $0C
TEBU 	= $0D

;
; clock defines
;
TEX 	= $0C
SPTR 	= $0D
SECC 	= $01
MIN 	= $02
HOUR	= $03
SSEC 	= $11
SMIN 	= $12
SHOUR = $13
QSEC 	= $0B
TIME 	= $9F
ALM		= $0E
ALH		= $0F
;
.org $F800
;
WAIT		STX TEMP 			; save x
 				LDX #$A0			; pause ca 800 microseconds
LOOP2		DEX 					; Increase readability display
 				BNE LOOP2			; determines beep frequency
 				LDX TEMP
				RTS 						
;
BEEP 		LDA KCODE			; beep as confirmation
 				BEQ RET1 			; key pressed
 				LDA DRB 			; 
 				EOR #$10 			; invert pb4 
 				STA DRB  
RET1 		RTS 
; 
TAB1 		AND #$0F 			; search display code
 				TAY 					; of low nibble 
 				LDA CODE1,Y 
 				STA DRA 			; send to display
 				JSR WAIT			; pause
 				RTS 
;
TAB2 		LSR A 				; search display code
				LSR A 				; of high nibble 
 				LSR A
 				LSR A
 				TAY 					;
 				LDA CODE1,Y 
 				STA DRA 			; send to display
 				JSR WAIT 			; pause
 				RTS 
;
DIS			CLD 					; Binary mode
				LDA #$00			; keycode 0
				STA KCODE			;
SCAN		LDA DRB				; DRB 01x10000
				AND #$70			; (leave data/adres led unchanged)
				ORA #$50			;
				STA DRB				;
				LDA #$FF			; port B output
				STA DDRB 			;
				LDX #$03			; x = 3
LOOP1		LDA #$FF			; port A outpout
				STA DDRA			;
				LDA $00,X			; adres/data register in A
				JSR TAB2			; display
				JSR BEEP			; and make sound
				LDA #$00			; prevent overflow
				STA DRA				; to next display
				INC DRB				; next display cathode
				LDA $00,X			; 
				JSR TAB1			; Show next address/data symbol
				JSR BEEP			; 
				LDA #$00			; 
				STA DRA				; prevent overflow
				STA DDRA			; port A inout
				INC DRB				;	next display
				DEX						;
				BNE LOOP1			;
				LDX #$04			; if x = 0 check keyboard
NCOL		LDA DRA				;
				EOR #$FF			; invert column keycodes
				BNE STORE			; remember keypressed 
				INC DRB				; next column
				DEX						; 
				BNE NCOL			; last column?
				LDA KCODE			; if key pressed switch off 
				BEQ SCAN			; 
				LDA DRB				; DRB 01x11111
				AND #$7F			;
				ORA #$5F			;
				STA DRB				;
				RTS						;
;
STORE		ASL A					; low nibble to high nibble, low nibble 0
				ASL A					;
				ASL A					;
				ASL A					;
				STA KCODE			; 
				LDA DRB				; column number in low nibble
				AND #$0F			;
				CLC						;
				ADC KCODE			;
				STA KCODE			;
				JMP SCAN			;
;
DIGIT		CLD						; binary mode 
				LDY #$00			; y = 0
LOOP5		LDA CODE2,Y		; lookup keycode in table
				CMP KCODE			; if yes then ADIG
				BEQ ADIG			;
				INY						;
				CPY #$10			;
				BNE LOOP5			; else return
				LDA #$00			;
				RTS						;
;
ADIG		LDA DRB				; 
				AND #$20			; data or address
				BEQ DDIG			; 
				LDA DPTR			; dptr 1
				CMP #$01			;
				BEQ NEW1			;
				BPL NCL1			; dptr > 1
				LDA #$00			; dptr = 0
				STA ADHI			;
				STA ADLO			;
NEW1		LDA #$05			; digitpointer 5
				STA DPTR			; 
NCL1		LDA ADHI			; shift 3rd digit to 4th position
				ASL A					;
				ASL A					;
				ASL A					;
				ASL A					;
				STA ADHI			;
				LDA ADLO			; 
				LSR A					; shift 2nd digit to 3rd position
				LSR A					;
				LSR A					;
				LSR A					;
				AND #$0F			;
				CLC						;
				ADC ADHI			;
				STA ADHI			;
				LDA ADLO			;
				ASL A					; shift 1st digit to second position
				ASL A					;
				ASL A					;
				ASL A					;
				STA ADLO			;
				TYA						; new digit in 1st position
				CLC						;
				ADC ADLO			;
				STA ADLO			;
				DEC DPTR			;
				LDA #$FF			;
				RTS 					;
;
;
;
DDIG		LDA DPTR			; 
				CMP #$01			; dptr = 1?
				BEQ NEW2			; dptr > 1
				BPL NCL2			; dptr = 0 
				LDA #$00			;
				STA DATA			; data reg 0
NEW2		LDA #$03			;
				STA DPTR			; digit pointer 3
NCL2		LDA DATA			; shift first digit to
  			ASL A					;  second position
  			ASL A					;
  			ASL A					;
  			ASL A					;
				STA DATA			; 
				TYA						; new digit to first position
				CLC						;
				ADC DATA			;
				STA DATA			;
				DEC DPTR			; decrement digit pointer
				LDA #$FF			; A = $FF
				RTS  					;
;
INCAD		CLD						; binary
				CLC						;
				LDA ADLO			; increment 
				ADC #$01			;
				STA ADLO			;
				LDA	ADHI			;
				ADC	#$00			;
				STA ADHI			;
				RTS						;
;
; Message
;
MSAGE		LDA #$00			; Keycode 0
				STA KCODE			;
STRT1		LDA #%01110000
				STA DRB				;
				LDA #$FF			; port B output
				STA DDRB			;
				STA DDRA			; port A output
				LDY LET1			; Y first character
LOOP3		LDA CODE1,Y		; 
				STA DRA				; char to port A
				JSR BEEP			;
				JSR WAIT			; beep and wait
				INY						; next char
				INC DRB				;
				CPY LLET			; last char?
				BNE LOOP3			;
				LDA #$00			; port A input
				STA DDRA			;
				LDA #%01110110; 
				STA DRB				;
				LDA DRA				;
				CMP #$EF			; Go key pressed?
				BEQ BPON			;
				LDA KCODE			; key code 0?
				BEQ STRT1			; again
				LDA #%01110000;
				STA DRB				; Go key pressed
				RTS						;
BPON		LDA #$06			; sound on
				STA KCODE			;
				JMP STRT1			;
;
;
;
KBDIS1	LDA #%01110000; port B turn address led on
				STA DRB				;
AGAIN		LDY #$00			;
				LDA #$00			; 
				STA DPTR			; digitpointer = 0
				LDA (ADLO),Y	; 
				STA DATA			; refresh data register
CON			JSR DIS				; show
				JSR DIGIT			;
				BEQ CMAND1		; command key?
				LDY #00				;
				LDA DRB				;
				AND #$20			; data led on?
				BNE CMDR			;
				LDA DATA			; refresh memory location
				STA (ADLO),Y	;
				LDA (ADLO),Y	; check if refreshed
				CMP DATA			;
				BEQ CON				;
				LDA #$24			; show 'ERR' if failed
				STA LET1			;
				LDA #$27			;
				STA LLET			;
				JSR MSAGE			;
				JMP AGAIN			;
CMDR		LDA (ADLO),Y	; if data led on
				STA DATA			; refresh data register
				JMP CON				; 
CMAND1	LDA KCODE			; process commands
				CMP #$09			; address key?
				BNE D1				;
				LDA #%01110000; turn on address led 
				STA DRB				;
				JMP AGAIN			;
D1			CMP #$08 			;
				BNE M1				;
				LDA #%01010000;	turn on dataled 
				STA DRB				;
				JMP AGAIN			;
M1			CMP #$07			; M key??
				BNE G1				;
				JSR INCAD			; add 1 to address
				JMP AGAIN			;
G1			CMP #$06			; GO key?
				BNE CON				;
				RTS						;
; 
; Monitor
;
BMON		LDX #$7F			; initialize stack
				TXS						; 
				LDA #$E2			; NMI and IRQ vectors to break routine
				STA IRQLO			;
				LDA #$F9			;
				STA IRQHI			;
				LDA #$BB			;
				STA NMILO			;
				LDA #$F9			; 
				STA NMIHI			;
				LDA #$2F			; show 'BETA 84'
				STA LET1			;
				LDA #$35			;
				STA LLET			;
				JSR MSAGE			;
				JSR KBDIS1		; handle keyboard and display
				JMP (ADLO)		; start user program
;
; Break  
;
BREAK		JMP (IRQLO)		; IRQ soft vector
				STA IACC			;  save A,X,Y, SP
				STX IXX				;
				STY IYY				;
				TSX						;
				STX ISS				;
				LDA #$00			; save overflow and carry in OC register
				BVC OVC				;
				LDA #$10			;
OVC			BCC CC				;
				ADC #$00			;
CC			STA IFLAG			;
				PLA						; save PC - 1
				STA TEMP			;
				PLA						;
				CLD						;
				SEC						;
				SBC #$1				;
				PHA						;
				STA IPCL			;
				LDA TEMP			;
				SBC #$00			;
				PHA						;
				LDA DDRA			; save port A and B data and direction
				STA IDDRA			;
				LDA DRA				;
				STA IDRA			;
				LDA DDRB			;
				STA IDDRB			;
				LDA DRB				;
				STA IDRB			;
				LDA #$0B			;
				STA ADLO			;
				LDA #$00			;
				STA ADHI			;
				JSR KBDIS1		; do display and reading keyboard
				LDA IDRA			;
				STA DRA				; if GO, remove port A and B modifications
				LDA IDDRA			;
				STA DDRA			;
				LDA IDRB			;
				STA DRB				;
				LDA IDDRB			;
				STA DDRB			;
				LDX IXX				;
				LDY IYY				;
				LDA IACC			;
				RTI						; resume from interrupt
;								
; Shift							
;				
SHIFT		LDA #$00			; port A ipout
				STA DDRA			;
				LDA #$7F			; port B 01111111
				STA DRB				;
				LDA #$FF			; port B outout
				STA DDRB			;
				LDA ADLO			;
				STA SADLO			;
				LDA ADHI			; refresh shift address
				STA SADHI			;
				LDY #$08			; 8 to shift 
SHI			LDA SADHI			;
				JSR SHIN			;
				ASL SADHI			;
				DEY						;
				BNE SHI				;
				LDY #$08			; 8 to shift low part
SLO			LDA SADLO			;
				JSR SHIN			;
				ASL SADLO			;
				DEY						;
				BNE SLO				;
				RTS						;
;
; Shift in
;
SHIN		AND #$80			;
				BNE HIBI			; msb 1?
				LDA #$57			;
				STA DRB				;
				LDA #$77			;
				STA DRB				;
				RTS						;
HIBI		LDA #$5F			; shift 1 in
				STA DRB				;
				LDA #$77			;
				STA DRB				;
				RTS						;
;				
; Read EPROM
;			
REPROM	LDA DRB				; save port B
				STA TEMP			;
				JSR SHIFT			; address to shift register
				LDA #$3F			;
				STA DRB				; 
				LDA DRA				; read EPROM
				STA DATA			; save in DATA
				LDA TEMP			; restore port B 
				STA DRB				;
				RTS						;
;											
; KBDIS2
;
KBDIS2	LDA DRB				; Port B 01x10000
				AND #%01110000;
				ORA #$50			;
				STA DRB				;
CDPTR		LDA #$00			; digit pointer = 0
				STA DPTR			;
REP			JSR REPROM			; read eprom data
LOOP4		JSR DIS				; read keyboard, display data
				JSR DIGIT			;
				BEQ CMAND2		; command key?
				LDA DRB				;
				AND #$20			;
				BEQ LOOP4			; data led on? then continue
				JMP REP				;
CMAND2	LDA KCODE			;
				CMP #$09			; address key?
				BNE D2				;
				LDA DRB				;
				ORA #$20			; show address led
				STA DRB				;
				JMP CDPTR			; 
D2			CMP #$08			; Data key?
				BNE M2				;
				RTS						;
M2			CMP #$07			; if not M Key loop
				BNE CDPTR			;
				RTS						;
;
; Read
;				
READ		LDA #$20			; show "READ'
				STA LET1			;
				LDA #$24			;
				STA LLET			;
				JSR MSAGE			;
START2	JSR KBDIS2		; read key, show display
				LDA KCODE			;
				CMP #$07			; If M key then increment address register
				BNE START2		;
				JSR INCAD			;
				JMP START2		;
;
; Check EPROM empty
;
FF			LDA #$27			; show "FF"
				STA LET1			;
				LDA #$2B			;
				STA LLET			;
				JSR MSAGE		;
				LDA #$00			; address register = 0
				STA	ADLO			;
				STA ADHI			;
RPRO		JSR REPROM		; check EPROM data FF ?
				LDA DATA			;
				CMP #$FF			;
				BEQ NOER			; if not show error
				LDA #$24			;
				STA LET1			;
				LDA #$27			;
				STA LLET			;
				JSR MSAGE			;
				JMP START2		;
NOER		JSR INCAD			; next EPROM address
				LDA ADLO			;
				BNE RPRO			;
				LDA ADHI			;
				CMP #$10			; loop until Address $1000
				BNE RPRO			;
				JMP BMON			; Jump to monitor

;
; Burn EPROM
;
BURN		LDA #$2B			; Show "BURN"
				STA LET1			;
				LDA #$2F			;
				STA LLET			;
				JSR MSAGE			;
NEXT		JSR KBDIS2		; read keyboard, show display
				LDA KCODE			;
				CMP #$08			; DATA key?
				BNE M3				;
				LDA DRB				;
				AND #$DF			; address to shift register
				STA DRB				;
				JMP NEXT			;
M3			CMP #$07			;
				BNE NEXT			;
				JSR SHIFT			;
				LDA DATA			;
				STA TEBU			;
				STA DRA				; data to port A
				LDA #$FF			; port A output
				STA DDRA			; 
				LDA #%11011111; port B 
				STA DRB				;
				LDA #$32			; delay 50 ms EPROM type 2716
TLOOP		LDX #$C6			;
NYET		DEX						;
				BNE	NYET			;
				SEC						;
				SBC #$01			;
				BNE TLOOP			;
				LDA #$5F			; check data written
				STA DRB				;
				JSR REPROM		;
				LDA DATA			;
				CMP TEBU			;
				BNE ERR				; data not verified
				JSR INCAD			; next address and loop 
				JMP NEXT			;
ERR			LDA #$24			; show "ERR"
				STA LET1			;
				LDA #$27			;
				STA LLET 			;
				JSR MSAGE			;
				JMP NEXT			;
;
; Soft vector NMI
;
JNMI		JMP (NMILO)		; jump via NMI soft vector
;
; Interrupt service
; alarm clock
;
INT			PHA						; save registers 
				STX TEX				;
				LDX #$17			; delay 1/4 second
TINT		DEX						; if Xtal is exactly 4 MHz
				BNE TINT			;
				LDA #$F4			;
				STA TIME			;
				SED						;
				LDX SPTR			;
				LDA QSEC			;
				CMP #$03			; 1/4 second passed?
				BEQ FSEC			;
				CLC						;
				ADC #$01			; next 1/4 second
				STA QSEC			;
				JMP ALARM			;
FSEC		LDA #$00			;
				STA QSEC			;
				LDA SECC,X		; next second
				CMP #$59			; 1/4 sec counter = 0
				BEQ FMIN			;
				CLC						;
				ADC #$01			;
				STA SECC,X		;
				JMP ALARM			;
FMIN		LDA #$00			;	next minute
				STA SECC,X		; sec = 0
				LDA MIN,X			;
				CMP #$59			;
				BEQ	FHOUR			;
				CLC						;
				ADC #$1				;
				STA MIN,X			;
				JMP ALARM			;
FHOUR		LDA #$00			; next hour
				STA MIN,X			; min = 0
				LDA HOUR,X		;
				CMP #$24			;
				BEQ FDAY			;
				CLC						;
				ADC #$01			;
				STA HOUR,X		;
				JMP ALARM			;
FDAY		LDA #$01			; next day
				STA HOUR,X		; hour = 0
ALARM		LDA HOUR			;
				CMP ALH				; is it time for alarm?
				BNE NOAL			;
				LDA MIN				;
				CMP ALM				;
				BNE NOAL			;
				LDA DRB				;
				AND #$20			; alarm active?
				BNE NOAL			;
				LDA #$FF			; beep!
				STA KCODE			;
NOAL		LDX TEX				;
				PLA						; restore registers and return from interrupt
				RTI						;
;
				NOP
; 24h alarm clock
;
CLOCK	  LDA #INT & $00FF                           
				STA 	IRQLO		; IRQ soft vector to INT rout
				LDA #INT /$0100	                           
				STA IRQHI			;                            
				LDA #$00			;                            
				STA QSEC			; set 1/4 sec, sec, min, hour
				STA SECC			;                            
				STA MIN				;                            
				STA HOUR			;                            
				STA SPTR			;                            
				LDA #$F4			;                            
				STA TIME			;                            
				CLI						; allow interrupts           
DISTI		LDA #$00			; keycode 0                  
				STA KCODE			;                            
WCMD		JSR SCAN			; display and get key        
				LDA KCODE			;                            
				CMP #$07			; M key?                     
				BNE ASET			;                            
				LDA #%01110000;                            
				STA DRB				;                            
				LDA #$10			; service prointer = $10     
				STA SPTR			;                            
				LDA #$00			;                            
				STA SECC			;                            
				STA DPTR			;                            
SETT		JSR DIS				; display and get key to set 
				JSR DIGIT			;                            
				BNE SETT			;                            
				LDA #$00			;	                           
				STA SPTR			;                            
				JMP DISTI			; show clock                 
ASET		CMP #$09			; address key?               
				BNE ALON			;                            
				LDA #%01110000;                            
				STA DRB				;                            
				LDA SECC			;                            
				STA SSEC			;                            
				LDA MIN				; save time                           
				STA SMIN			;                            
				LDA HOUR			;                            
				STA SHOUR			;                            
				LDA #$10			;                            
				STA SPTR			;                            
				LDA #$00			;                            
				STA SECC			;                            
				STA DPTR			;                            
				LDA ALH				; alarm time on display      
				STA HOUR			;                            
				LDA ALM				;                            
				STA MIN				;                            
SETA		JSR DIS				; show display and get key   
				JSR DIGIT			;                            
				BNE SETA			;                            
				LDA MIN				;                            
				STA ALM				;                            
				LDA HOUR			;                            
				STA ALH				;                            
				LDA SSEC			; restore time                          
				STA SECC			;                            
				LDA SMIN			;                            
				STA MIN				;                            
				LDA SHOUR			;                            
				STA HOUR			;                            
				LDA #$00			;                            
				STA SPTR			;                            
				JMP DISTI			;                            
ALON		CMP #$08			; Data key?                  
				BNE ALOFF			;                            
				LDA #%01010000;	alarm on                   
				STA DRB				;                            
				JMP DISTI			;                            
ALOFF		CMP #$06			; Go key?                    
				BNE WCMD			;                            
				LDA #%01110000; alarm off                  
				STA DRB				;                            
				JMP DISTI			;                            
;                                                  
				.ORG $FFC5
;
CODE1		.BYTE $77,$24,$6B,$6D
				.BYTE $3C,$5D,$5F,$64
				.BYTE $7F,$7D,$7E,$1F
				.BYTE $53,$2F,$5B,$5A
;		
; keycode table
; 
CODE2		.BYTE $17,$26,$27,$28
				.BYTE $46,$47,$48,$86
				.BYTE $87,$88,$16,$18
				.BYTE $19,$29,$49,$89
				.BYTE $52,$5B,$7E,$2F
				.BYTE $5B,$0A,$0A,$5A
				.BYTE $5A,$00,$6A,$1F 
				.BYTE $07,$0A,$0E,$1F
				.BYTE $5B,$1B,$7E,$7F
				.BYTE $3C
;
; vectors
;
				.org $FFFA
;
NMI			.BYTE JNMI & $00FF
        .BYTE JNMI / $0100 
RESET		.BYTE BMON & $00FF
        .BYTE BMON / $0100
IRQ     .BYTE BREAK & $00FF
        .BYTE BREAK / $0100
;
		
 .END